/**
 * This work was created by participants in the DataONE project, and is
 * jointly copyrighted by participating institutions in DataONE. For
 * more information on DataONE, see our web site at http://dataone.org.
 *
 *   Copyright 2014
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.dataone.tidy.dao;

import java.util.List;
import java.util.Map;

import org.dataone.cn.dao.exceptions.DataAccessException;
import org.dataone.service.types.v1.Identifier;
import org.dataone.service.types.v1.NodeReference;
import org.dataone.service.types.v1.SystemMetadata;

/**
 * A specification for accessing and manipulating system metadata attributes in a database
 * 
 * @author cjones
 *
 */
public interface SystemMetadataTidyDao {

    /**
     * List all of the identifiers found in all CN identifier tables in the database
     * @return
     * @throws DataAccessException
     */
    public List<Identifier> listIdentifiers() throws DataAccessException;

    /**
     * Return system metadata objects from the three CNs for a given pid.  For a given entry,
     * the SystemMetadata object may be empty if it was not found on the CN
     * 
     * @return  a map of system metadata objects keyed by node identifier
     * @throws DataAccessException
     */
    public Map<NodeReference, SystemMetadata> getSystemMetadata(Identifier identifier)
            throws DataAccessException;

    /**
     * For a given identifier, save the system metadata to the merged database tables.
     * 
     * @param systemMetadata
     * @throws DataAccessException
     */
    public void saveSystemMetadata(SystemMetadata systemMetadata) throws DataAccessException;
}
