/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package org.dataone.tidy.concurrent;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.TimeUnit;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dataone.service.types.v1.Identifier;
import static org.junit.Assert.assertFalse;
import org.junit.Test;
import static org.junit.Assert.assertTrue;
/**
 *
 * @author waltz
 */
public class TidyJobThreadPoolExecutorTest {

    static final int corePoolSize = 1;
    static final int maximumPoolSize = 2;
    static final long keepAliveTime = 2L;

    private BlockingQueue workQueue = new LinkedBlockingQueue();
    TidyJobFactory mockTidyJobFactory = new TidyJobFactory(null, null, null, null);
    MockFailureTidyJob mockFailureTidyJobInstance = new MockFailureTidyJob();
    MockSuccessfulTidyJob mockSuccessfulTidyJobInstance = new MockSuccessfulTidyJob();

    private static Log logger = LogFactory.getLog(TidyJobThreadPoolExecutorTest.class);
    @Test
    public void TidyJobSuccessfulTest() throws InterruptedException, ExecutionException {
        logger.debug("starting");
        TidyJobFactory.setTinyJobClassName(MockSuccessfulTidyJob.class.getCanonicalName());
        TidyJobThreadPoolExecutor mergeJobExecutor = new TidyJobThreadPoolExecutor(corePoolSize, maximumPoolSize, 
                keepAliveTime, TimeUnit.SECONDS, workQueue);
        Identifier pid1 = new Identifier();
        pid1.setValue("sucessfulPid");
        TidyJob tidyJob = mockTidyJobFactory.getTidyJob(pid1);
        Future<Boolean> future = mergeJobExecutor.submit(tidyJob);
        
        assertTrue(future.get());
        mergeJobExecutor.shutdownNow();
        while (!mergeJobExecutor.isShutdown()) {
            try {
                Thread.sleep(100L);
            } catch (InterruptedException ex) {
                logger.error(ex); 
            }
        }
        logger.debug("ending");
    }
    @Test(expected=ExecutionException.class)
    public void TidyJobFailureTest() throws InterruptedException, ExecutionException {
        logger.debug("starting");
        TidyJobFactory.setTinyJobClassName(MockFailureTidyJob.class.getCanonicalName());
        TidyJobThreadPoolExecutor mergeJobExecutor = new TidyJobThreadPoolExecutor(corePoolSize, maximumPoolSize, 
                keepAliveTime, TimeUnit.SECONDS, workQueue);
        Identifier pid1 = new Identifier();
        pid1.setValue("FailedPid");
        TidyJob tidyJob = mockTidyJobFactory.getTidyJob(pid1);
        Future<Boolean> future = mergeJobExecutor.submit(tidyJob);
        try {
            Boolean futureRtn = future.get();
        } catch (ExecutionException e) {
            // this thread needs to sleep so that the assert statements are not called 
            // before the afterExecute is called in the mergeJobExecutor serviceS
            Thread.sleep(250L);
            logger.debug(e);
            logger.debug("failure test " + e.getCause());
            assertFalse(mergeJobExecutor.getFailures().isEmpty());
            assertTrue(mergeJobExecutor.getFailures().contains(tidyJob));
           mergeJobExecutor.shutdownNow();
            while (!mergeJobExecutor.isShutdown()) {
                try {
                    Thread.sleep(100L);
                } catch (InterruptedException ex) {
                    logger.error(ex); 
                }
            }
            logger.debug("ending");
            throw e;
        } 
        
        
 
    }
}
