/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package org.dataone.tidy.merge;

import static org.junit.Assert.assertTrue;
import junit.framework.Assert;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dataone.cn.dao.exceptions.DataAccessException;
import org.dataone.configuration.Settings;
import org.dataone.mock.tidy.dao.MockEmptyMergeResultRepository;
import org.dataone.mock.tidy.dao.MockKnbMergeResultRepository;
import org.dataone.mock.tidy.dao.MockKnbSystemMetadataTidyDao;
import org.dataone.mock.tidy.dao.MockSimpleSystemMetadataTidyDao;
import org.dataone.tidy.concurrent.MockExecutionTimeoutTidyJob;
import org.dataone.tidy.concurrent.MockFailureTidyJob;
import org.dataone.tidy.concurrent.MockFatalTidyJob;
import org.dataone.tidy.concurrent.MockSuccessfulTidyJob;
import org.dataone.tidy.concurrent.MockTimeoutTidyJob;
import org.dataone.tidy.concurrent.TidyJobFactory;
import org.dataone.tidy.dao.ChangeRecordRepository;
import org.dataone.tidy.dao.MergeResultRepository;
import org.dataone.tidy.dao.SystemMetadataTidyDao;
import org.junit.BeforeClass;
import org.junit.Test;

/**
 *
 * @author waltz
 */
public class MergeExecutorServiceTest {
    private static Log logger = LogFactory.getLog(MergeExecutorServiceTest.class);

    @BeforeClass
    public static void initTest() {
        Integer maxTimoutCount = 3;
        Long futureTimeoutSeconds = 1L;
        Settings.getConfiguration().addProperty("tidy.merge.executor.queue.min.size", 5);

        Settings.getConfiguration().addProperty("tidy.merge.executor.queue.max.size", 10);
        Settings.getConfiguration().addProperty("tidy.merge.executor.future.timeout.count.max",
                maxTimoutCount);

        Settings.getConfiguration().addProperty("tidy.merge.executor.future.timeout.seconds",
                futureTimeoutSeconds);

    }

    @Test
    public void executeSuccessfulMergeExecuturServiceTest() throws DataAccessException {
        SystemMetadataTidyDao systemMetadataDao = new MockKnbSystemMetadataTidyDao();
        MergeResultRepository mergeResultRepository = new MockEmptyMergeResultRepository();

        ChangeRecordRepository changeRecordRepository = null;
        MNSystemMetadataDelegate SystemMetadataDelegate = null;
        TidyJobFactory mockTidyJobFactory = new TidyJobFactory(systemMetadataDao,
                mergeResultRepository, changeRecordRepository, SystemMetadataDelegate);
        TidyJobFactory.setTinyJobClassName(MockSuccessfulTidyJob.class.getCanonicalName());
        MergeExecutorService testMergeExecutorService = new MergeExecutorService(systemMetadataDao,
                mergeResultRepository, mockTidyJobFactory);
        testMergeExecutorService.executeJobs();
        logger.info(testMergeExecutorService.getTotalToProcess() + " "
                + testMergeExecutorService.getTotalCompletedJobs() + " "
                + testMergeExecutorService.getTotalFailedJobs());
        assertTrue(testMergeExecutorService.getTotalToProcess() == (testMergeExecutorService
                .getTotalCompletedJobs() + testMergeExecutorService.getTotalFailedJobs()));
    }

    @Test
    public void executeSuccessfulMergeExecuturWithMergeResultsServiceTest()
            throws DataAccessException {

        SystemMetadataTidyDao systemMetadataDao = new MockKnbSystemMetadataTidyDao();
        MergeResultRepository mergeResultRepository = new MockKnbMergeResultRepository();

        ChangeRecordRepository changeRecordRepository = null;
        MNSystemMetadataDelegate SystemMetadataDelegate = null;
        TidyJobFactory mockTidyJobFactory = new TidyJobFactory(systemMetadataDao,
                mergeResultRepository, changeRecordRepository, SystemMetadataDelegate);
        TidyJobFactory.setTinyJobClassName(MockSuccessfulTidyJob.class.getCanonicalName());
        MergeExecutorService testMergeExecutorService = new MergeExecutorService(systemMetadataDao,
                mergeResultRepository, mockTidyJobFactory);
        testMergeExecutorService.executeJobs();
        logger.debug(testMergeExecutorService.getTotalToProcess() + " "
                + testMergeExecutorService.getTotalCompletedJobs() + " "
                + testMergeExecutorService.getTotalFailedJobs());

        // Starts with 25 total jobs.  Prior completed returns 11 mergeResults.  Total to process is then 14.
        Assert.assertEquals(14, testMergeExecutorService.getTotalToProcess().intValue());

        assertTrue(testMergeExecutorService.getTotalToProcess() == (testMergeExecutorService
                .getTotalCompletedJobs() + testMergeExecutorService.getTotalFailedJobs()));
    }

    @Test
    public void executeFailedMergeExecuturServiceTest() throws DataAccessException {

        SystemMetadataTidyDao systemMetadataDao = new MockKnbSystemMetadataTidyDao();
        MergeResultRepository mergeResultRepository = new MockEmptyMergeResultRepository();

        ChangeRecordRepository changeRecordRepository = null;
        MNSystemMetadataDelegate SystemMetadataDelegate = null;
        TidyJobFactory mockTidyJobFactory = new TidyJobFactory(systemMetadataDao,
                mergeResultRepository, changeRecordRepository, SystemMetadataDelegate);
        TidyJobFactory.setTinyJobClassName(MockFailureTidyJob.class.getCanonicalName());
        MergeExecutorService testMergeExecutorService = new MergeExecutorService(systemMetadataDao,
                mergeResultRepository, mockTidyJobFactory);
        testMergeExecutorService.executeJobs();
        logger.info(testMergeExecutorService.getTotalToProcess() + " "
                + testMergeExecutorService.getTotalCompletedJobs() + " "
                + testMergeExecutorService.getTotalFailedJobs());
        assertTrue(testMergeExecutorService.getTotalToProcess() == (testMergeExecutorService
                .getTotalCompletedJobs() + testMergeExecutorService.getTotalFailedJobs()));
    }

    @Test
    public void executeTimeoutMergeExecuturServiceTest() throws DataAccessException {

        SystemMetadataTidyDao systemMetadataDao = new MockKnbSystemMetadataTidyDao();
        MergeResultRepository mergeResultRepository = new MockEmptyMergeResultRepository();

        ChangeRecordRepository changeRecordRepository = null;
        MNSystemMetadataDelegate SystemMetadataDelegate = null;
        TidyJobFactory mockTidyJobFactory = new TidyJobFactory(systemMetadataDao,
                mergeResultRepository, changeRecordRepository, SystemMetadataDelegate);
        TidyJobFactory.setTinyJobClassName(MockTimeoutTidyJob.class.getCanonicalName());
        MergeExecutorService testMergeExecutorService = new MergeExecutorService(systemMetadataDao,
                mergeResultRepository, mockTidyJobFactory);
        testMergeExecutorService.executeJobs();
        logger.info(testMergeExecutorService.getTotalToProcess() + " "
                + testMergeExecutorService.getTotalCompletedJobs() + " "
                + testMergeExecutorService.getTotalFailedJobs());
        assertTrue(testMergeExecutorService.getTotalToProcess() == (testMergeExecutorService
                .getTotalCompletedJobs() + testMergeExecutorService.getTotalFailedJobs()));
    }

    @Test
    public void executeExecutionTimeoutMergeExecuturServiceTest() throws DataAccessException {

        SystemMetadataTidyDao systemMetadataDao = new MockKnbSystemMetadataTidyDao();
        MergeResultRepository mergeResultRepository = new MockEmptyMergeResultRepository();

        ChangeRecordRepository changeRecordRepository = null;
        MNSystemMetadataDelegate SystemMetadataDelegate = null;
        TidyJobFactory mockTidyJobFactory = new TidyJobFactory(systemMetadataDao,
                mergeResultRepository, changeRecordRepository, SystemMetadataDelegate);
        TidyJobFactory.setTinyJobClassName(MockExecutionTimeoutTidyJob.class.getCanonicalName());
        MergeExecutorService testMergeExecutorService = new MergeExecutorService(systemMetadataDao,
                mergeResultRepository, mockTidyJobFactory);
        testMergeExecutorService.executeJobs();
        logger.info(testMergeExecutorService.getTotalToProcess() + " "
                + testMergeExecutorService.getTotalCompletedJobs() + " "
                + testMergeExecutorService.getTotalFailedJobs());
        assertTrue(testMergeExecutorService.getTotalToProcess() == (testMergeExecutorService
                .getTotalCompletedJobs() + testMergeExecutorService.getTotalFailedJobs()));
    }

    @Test
    public void executeFatalMergeExecuturServiceTest() throws DataAccessException {

        SystemMetadataTidyDao systemMetadataDao = new MockSimpleSystemMetadataTidyDao();
        MergeResultRepository mergeResultRepository = new MockEmptyMergeResultRepository();

        ChangeRecordRepository changeRecordRepository = null;
        MNSystemMetadataDelegate SystemMetadataDelegate = null;
        TidyJobFactory mockTidyJobFactory = new TidyJobFactory(systemMetadataDao,
                mergeResultRepository, changeRecordRepository, SystemMetadataDelegate);
        TidyJobFactory.setTinyJobClassName(MockFatalTidyJob.class.getCanonicalName());
        MergeExecutorService testMergeExecutorService = new MergeExecutorService(systemMetadataDao,
                mergeResultRepository, mockTidyJobFactory);
        testMergeExecutorService.executeJobs();
        logger.info(testMergeExecutorService.getTotalToProcess() + " "
                + testMergeExecutorService.getTotalCompletedJobs() + " "
                + testMergeExecutorService.getTotalFailedJobs());
        assertTrue(testMergeExecutorService.getTotalCompletedJobs() == 0
                && testMergeExecutorService.getTotalFailedJobs() == 0);
    }
}
